<?php
/*
Plugin Name: fake sản phẩm đã bán
Plugin URI: https://themewpgiare.com
Description: Display Progress sold for WooCommerce - Hiển thị số lượng sản phẩm đã bán cho WooCommerce
Version: 1.0
Author: Trường Phong
Author URI: https://themewpgiare.com
*/

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );
if(!function_exists('add_action')) {
	echo "Hi there!  I'm just a plugin, not much I can do when called directly.";
	exit;
}

if (!function_exists('is_plugin_active')) {
    include_once(ABSPATH . 'wp-admin/includes/plugin.php');
}

require_once('common.php');

define('PK_PROGRESS_SOLD_PLUGIN_URL', plugin_dir_url(__FILE__));

if(is_plugin_active('woocommerce/woocommerce.php')) {

	if(!class_exists('PK_PROGRESS_SOLD')) {

		class PK_PROGRESS_SOLD
		{
			private $_version = '1.0';
			private $_plugin_slug = 'pk-progress-sold';
			private $_option_name = 'pk_progress_sold_option';

			public function __construct()
			{
				$this->pk_progress_sold_option = get_option($this->_option_name);
				add_action('admin_menu', [$this, 'add_settings_page']);
				add_action('admin_init', [$this, 'setup_field']);
				add_filter('plugin_action_links_' . plugin_basename(__FILE__), [$this, 'add_setting_link'] );

				add_action('wp_enqueue_scripts', array($this, 'pk_load_plugins_scripts'));

				add_action($this->pk_progress_sold_option['pk_position'], array($this, 'pk_html_progress_sold'));

				add_action('activated_plugin', array($this, 'pk_activation_redirect'));

				add_action('woocommerce_product_options_general_product_data', array($this, 'woocommerce_product_custom_fields'));
				add_action('woocommerce_process_product_meta', array($this, 'woocommerce_product_custom_fields_save'));

				add_action('admin_enqueue_scripts', function(){
					wp_register_script('pk_progress_sold_js', PK_PROGRESS_SOLD_PLUGIN_URL . 'custom.js', array('jquery'));
					wp_localize_script('pk_progress_sold_js', 'pk_progress_sold_js', [
						'url' => admin_url('admin-ajax.php')
					]);
					wp_enqueue_script('pk_progress_sold_js');
				});

				add_action('wp_ajax_create_data_fake', [$this, 'create_data_fake']);
			}

			public function add_settings_page()
			{
				$parent_slug = 'options-general.php';
				$page_title = 'PK Progress Sold';
				$menu_title = 'PK Progress Sold';
				$capability = 'manage_options';
				$slug = $this->_plugin_slug;
				$callback = [$this, 'plugin_settings_page_content'];
			
				add_submenu_page($parent_slug, $page_title, $menu_title, $capability, $slug, $callback);
			}

			public function plugin_settings_page_content()
			{
				if(!current_user_can('manage_options')) return;
				?>
					<div class="wrap">
						<h2>FAKE PROGRESS SOLD - SỐ LƯỢNG SẢN PHẨM ĐÃ BÁN ẢO</h2>

						<form method="post" action="options.php">
							<?php
								settings_fields( $this->_plugin_slug );
								do_settings_sections( $this->_plugin_slug );
								submit_button();
							?>
						</form>

					</div>
				<?php
			}

			public function add_setting_link($links)
			{
				array_unshift($links, '<a href="' . admin_url('options-general.php?page='.$this->_plugin_slug) . '">' . __('Settings') . '</a>');
				return $links;
			}

			public function setup_field()
			{
				register_setting(
					$this->_plugin_slug, // Option group
					$this->_option_name, // Option name
					array( $this, 'sanitize' ) // Sanitize
				);

				add_settings_section(
					'pk_progress_sold_section', // ID
					'CẤU HÌNH', // Title
					false, // Callback
					$this->_plugin_slug // Page
				);

				$fields_section = [
					[
						'id'       => 'pk_progress_sold_title',
						'title'    => 'Tiêu đề',
						'callback' => 'pk_title_html'
					],
					[
						'id'       => 'pk_use_data_woo',
						'title'    => 'Sử dụng dữ liệu Woocommerce',
						'callback' => 'pk_use_data_woo_html'
					],
					[
						'id'       => 'pk_progress_sold_position',
						'title'    => 'Vị trí',
						'callback' => 'pk_position_html'
					],
					[
						'id'       => 'pk_button_fake_data',
						'title'    => 'Tạo dữ liệu Fake',
						'callback' => 'pk_button_fake_data_html'
					],
					[
						'id'       => 'pk_support',
						'title'    => 'Liên hệ hỗ trợ',
						'callback' => 'pk_support_html'
					]
				];

				foreach($fields_section as $field){
					add_settings_field(
						$field['id'],
						$field['title'],
						[$this, $field['callback']],
						$this->_plugin_slug,
						'pk_progress_sold_section' // ID Setting Section
					);
				}
			}

			public function sanitize($input)
			{
				$sanitized_input = [];

				foreach($input as $key => $item) {
					$sanitized_input[$key] = sanitize_text_field($item);
				}

				return $sanitized_input;
			}

			public function pk_title_html()
			{
				printf(
						'<input type="text" style="width:300px" id="pk_progress_sold_title" name="'.$this->_option_name.'[pk_progress_sold_title]" value="%s" placeholder="Đã bán" />',
						isset( $this->pk_progress_sold_option['pk_progress_sold_title'] ) ? esc_attr( $this->pk_progress_sold_option['pk_progress_sold_title']) : ''
					);
			}

			public function pk_use_data_woo_html()
			{
				echo '<input type="checkbox" id="pk_use_data_woo" name="'.$this->_option_name.'[pk_use_data_woo]" value="1" '. checked(1, isset($this->pk_progress_sold_option['pk_use_data_woo']) ? 1 : 0, false ) .' /> <i><small>(Sử dụng dữ liệu kho hàng và đơn hàng thực tế từ WooCommerce)</small></i>';
			}

			public function pk_position_html()
			{
				?>
				<select id="pk_position" name="<?php echo $this->_option_name.'[pk_position]' ?>">
					<option value="">Chọn vị trí</option>
				
					<option value="woocommerce_before_shop_loop_item_title" 
						<?php echo $this->pk_progress_sold_option['pk_position'] == 'woocommerce_before_shop_loop_item_title' ? 'selected' : null ?>>
						Dưới ảnh sản phẩm
					</option>
					<option value="woocommerce_shop_loop_item_title" 
						<?php echo $this->pk_progress_sold_option['pk_position'] == 'woocommerce_shop_loop_item_title' ? 'selected' : null ?>>
						Trên tên sản phẩm
					</option>
					<option value="woocommerce_after_shop_loop_item_title" 
						<?php echo $this->pk_progress_sold_option['pk_position'] == 'woocommerce_after_shop_loop_item_title' ? 'selected' : null ?>>
						Dưới tên sản phẩm
				
				</select>
				<?php
			}

			public function pk_button_fake_data_html() {
				echo '<a href="javascript:;" class="button pk_btn_fake_data">Tạo dữ liệu Fake</a>';
			}

			public function pk_support_html() {
				echo '<a href="https://themewpgiare.com"><i>https://themewpgiare.com</i></a>';
			}

			public function pk_html_progress_sold()
			{
				global $product;

				if(!isset($this->pk_progress_sold_option['pk_use_data_woo'])){
					$pk_product_total = get_post_meta($product->get_id(), '_pk_product_total', true);
					$pk_product_sold = get_post_meta($product->get_id(), '_pk_product_sold', true);
				} else {
					$pk_product_total = $product->get_stock_quantity();
					$pk_product_sold = get_post_meta($product->get_id(), 'total_sales', true);
				}

				if(!empty($pk_product_total)):

				$random = mt_rand();
				?>
					<div class="skill-set">
					    <div class="skill-set__flex">
					        <ul class="skill-set__list">
					            <li class="skill-set__item">
					                <div class="skill-set__detail">
					                    <div class="skill-set__bar percentage_bar_<?php echo $random ?>">
					                        <div class="skill-set_fire"></div>
					                        <span class="skill-set_percentage">
						                        <?php echo !empty( $this->pk_progress_sold_option['pk_progress_sold_title'] ) ? esc_attr( $this->pk_progress_sold_option['pk_progress_sold_title']) : 'Đã bán' ?> <?php echo $pk_product_sold ?>
						                    </span> 
					                    </div>
					                </div>
					            </li>
					        </ul>
					    </div>
					</div>

					<style>
					    .skill-set__bar.percentage_bar_<?php echo $random ?>:after {
					        width: <?php echo !empty($pk_product_total) && !empty($pk_product_sold) && $pk_product_total > 0 ? ($pk_product_sold/$pk_product_total) * 100 : 0 ?>%;
					    }
					</style>

				<?php
				endif;
			}

			public function pk_activation_redirect($plugin)
			{
			    if( $plugin == plugin_basename( __FILE__ ) ) {
			        exit(wp_redirect(admin_url('admin.php?page='. $this->_plugin_slug )));
			    }
			}

			public function pk_load_plugins_scripts()
			{
				wp_enqueue_style('pk-progress-sold-style', PK_PROGRESS_SOLD_PLUGIN_URL.'pk_progress_sold.css', array(), $this->_version, 'all');
			}

			public function woocommerce_product_custom_fields()
			{
				if(!isset($this->pk_progress_sold_option['pk_use_data_woo'])):
					echo '<div class="options_group">';
					
				    woocommerce_wp_text_input(
				        array(
				            'id' => '_pk_product_total',
				            'placeholder' => 'Số lượng trong kho',
				            'label' => __('Số lượng trong kho', 'woocommerce'),
				            'type' => 'number',
				            'custom_attributes' => array(
				                'step' => 'any',
				                'min' => '0'
				            )
				        )
				    );

				    woocommerce_wp_text_input(
				        array(
				            'id' => '_pk_product_sold',
				            'placeholder' => 'Số lượng đã bán (nhỏ hơn số lượng trong kho)',
				            'label' => __('Số lượng đã bán', 'woocommerce'),
				            'type' => 'number',
				            'custom_attributes' => array(
				                'step' => 'any',
				                'min' => '0'
				            )
				        )
				    );

					echo '</div>';
				endif;
			}

			public function woocommerce_product_custom_fields_save($post_id)
			{
				if(!isset($this->pk_progress_sold_option['pk_use_data_woo'])):
				    $pk_product_total = $_POST['_pk_product_total'];
				    $pk_product_sold = $_POST['_pk_product_sold'];

				    if($pk_product_total >= $pk_product_sold) {
				    	if (!empty($pk_product_total))
					        update_post_meta($post_id, '_pk_product_total', esc_attr($pk_product_total));

					    if (!empty($pk_product_sold))
					        update_post_meta($post_id, '_pk_product_sold', esc_attr($pk_product_sold));
				    }
				endif;
			}

			public function create_data_fake()
			{
				$all_ids = get_posts( array(
			        'post_type' => 'product',
			        'numberposts' => -1,
			        'post_status' => 'publish',
			        'fields' => 'ids'
			   	));

				foreach($all_ids as $id) {
					$quantity_total = rand(1, 100);

					$arr_per = [0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9];
					$per = $arr_per[array_rand($arr_per)];
					
					$quantity_sold = round($per * $quantity_total);

			    	update_post_meta($id, '_pk_product_total', esc_attr($quantity_total));
			    	update_post_meta($id, '_pk_product_sold', esc_attr($quantity_sold));
			   	}

				wp_send_json_success([
					'status'  => true,
					'message' => 'Tạo dữ liệu Fake thành công',
					'data'    => $all_ids
				]);
			}
		}

		new PK_PROGRESS_SOLD();

	}

} else {

	progress_sold_admin_notice('Require WooCommerce Plugin Active');

}